/* 
 * datebook.c: interface routines to the datebook
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 * $Id: datebook.c,v 1.1 1999/02/21 13:41:56 ga58 Exp $
 *
 * $Log: datebook.c,v $
 * Revision 1.1  1999/02/21 13:41:56  ga58
 *  - Version 3.29
 *
 * Revision 1.1  1999/01/17 20:02:14  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 */

#pragma pack(2)

#include "titraxAll.h"
#include "callback.h"
#include "datebook.h"

static DmOpenRef DatebookDB = 0;
static UInt usageCount = 0;

Int 
DateCompare(DateType d1, DateType d2)
{
  UInt int1, int2;
	
  int1 = DateToInt(d1);
  int2 = DateToInt(d2);
	
  if (int1 > int2)
	 return (1);
  else if (int1 < int2)
	 return (-1);
  return 0;
}

Int 
TimeCompare(TimeType t1, TimeType t2)
{
  Int int1, int2;
	
  int1 = TimeToInt(t1);
  int2 = TimeToInt(t2);
	
  if (int1 > int2)
	 return (1);
  else if (int1 < int2)
	 return (-1);
  return 0;
}

void
DatebookOpenDB(void)
{
  usageCount++;

  if (DatebookDB == 0) {
	 DatebookDB = DmOpenDatabaseByTypeCreator('DATA', 'date', dmModeReadWrite);
	 if (DatebookDB == 0)
		ErrDisplay("Cannot open datebook database.");
  }
}

void
DatebookCloseDB(void)
{
  usageCount--;

  if (usageCount == 0) {
	 DmCloseDatabase(DatebookDB);
	 DatebookDB = 0;
  }
}

UInt 
DatebookRecordPackedSize(ApptDBRecordPtr r)
{
  UInt size;
	
  size = sizeof(ApptDateTimeType) + sizeof(ApptDBRecordFlags);
	
  if (r->alarm != NULL)
	 size += sizeof(AlarmInfoType);
	
  if (r->repeat != NULL)
	 size += sizeof(RepeatInfoType);
	
  if (r->exceptions != NULL)
	 size += sizeof(UInt) + 
		(r->exceptions->numExceptions * sizeof(DateType));
	
  if (r->description != NULL)
	 size += StrLen(r->description) + 1;
	
  if (r->note != NULL)
	 size += StrLen(r->note) + 1;

  return size;
}

void 
DatebookRecordPack(ApptDBRecordPtr s, ApptPackedDBRecordPtr d)
{
  UInt size;
  ULong offset = 0;
  ApptDBRecordFlags	flags;
	
  *(unsigned char *)&flags = 0;
	
  offset = 0;
  DmWrite(d, offset, s->when, sizeof(ApptDateTimeType));
  offset += sizeof(ApptDateTimeType) + sizeof(ApptDBRecordFlags);

  if (s->alarm != NULL) {
	 DmWrite(d, offset, s->alarm, sizeof(AlarmInfoType));
	 offset += sizeof (AlarmInfoType);
	 flags.alarm = 1;
  }

  if (s->repeat != NULL) {
	 DmWrite(d, offset, s->repeat, sizeof(RepeatInfoType));
	 offset += sizeof (RepeatInfoType);
	 flags.repeat = 1;
  }

  if (s->exceptions != NULL) {
	 size = sizeof (UInt) + 
		(s->exceptions->numExceptions * sizeof (DateType));
	 DmWrite(d, offset, s->exceptions, size);
	 offset += size;
	 flags.exceptions = 1;
  }
	
  if (s->description != NULL) {
	 size = StrLen(s->description) + 1;
	 DmWrite(d, offset, s->description, size);
	 offset += size;
	 flags.description = 1;
  }
	
  if (s->note != NULL) {
	 size = StrLen(s->note) + 1;
	 DmWrite(d, offset, s->note, size);
	 offset += size;
	 flags.note = 1;
  }
  
  DmWrite(d, sizeof(ApptDateTimeType), &flags, sizeof(flags));
}

Int 
DatebookRecordComparePackedRecords (ApptPackedDBRecordPtr r1, 
												ApptPackedDBRecordPtr r2, 
												Int extra, SortRecordInfoPtr info1, 
												SortRecordInfoPtr info2, VoidHand appInfoH)
{
  Int result;

  CALLBACK_PROLOGUE

  if ((r1->flags.repeat) || (r2->flags.repeat)) {
	 if ((r1->flags.repeat) && (r2->flags.repeat))
		result = 0;
	 else if (r1->flags.repeat)
		result = -1;
	 else
		result = 1;
  } else {
	 result = DateCompare(r1->when.date, r2->when.date);
	 if (result == 0) {
		result = TimeCompare (r1->when.startTime, r2->when.startTime);
	 }
  }
  
  CALLBACK_EPILOGUE
	 
  return result;
}

UInt 
DatebookRecordFindSortPosition(ApptPackedDBRecordPtr newRecord)
{
  return (DmFindSortPosition(DatebookDB, newRecord, NULL, (DmComparF *)DatebookRecordComparePackedRecords, 0));
}

void 
DatebookCreateRecord(ApptDBRecordPtr r)
{
  UInt newIdx;
  VoidHand recH;
  ApptPackedDBRecordPtr recP;

  recH = DmNewHandle(DatebookDB, (ULong)DatebookRecordPackedSize(r));
  if (recH == NULL)
	 ErrDisplay("DatebookCreateRecord: could not create record.");

  recP = MemHandleLock(recH);
  DatebookRecordPack(r, recP);
  newIdx = DatebookRecordFindSortPosition(recP);
  MemPtrUnlock(recP);

  if (DmAttachRecord(DatebookDB, &newIdx, (Handle)recH, 0)) {
	 MemHandleFree(recH);
	 ErrDisplay("DatebookCreateRecord: could not attach record.");
  }
}

