/* 
 * datebook.h: interface routines to the datebook
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 * $Id: datebook.h,v 1.1 1999/02/21 13:41:56 ga58 Exp $
 *
 * $Log: datebook.h,v $
 * Revision 1.1  1999/02/21 13:41:56  ga58
 *  - Version 3.29
 *
 * Revision 1.1  1999/01/17 20:02:14  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 */

#ifndef _datebook_h
#define _datebook_h


typedef struct {
   UInt	renamedCategories;	// bitfield of categories with a different name
	char 	categoryLabels[dmRecNumCategories][dmCategoryLength];
	Byte	categoryUniqIDs[dmRecNumCategories];
	Byte	lastUniqID;	// Uniq IDs generated by the device are between
							// 0 - 127.  Those from the PC are 128 - 255.
	Byte	reserved1;	// from the compiler word aligning things
	Word	reserved2;
	Byte	startOfWeek;
} ApptAppInfoType;

typedef ApptAppInfoType * ApptAppInfoPtr;

#define apptMaxPerDay 		100		// max appointments displayable on a day.
#define apptNoTime			-1			// start time of an untimed appt.
#define apptNoEndDate		-1			// end date of appts that repeat forever
#define apptNoAlarm			-1
#define apptMaxEndTime		0x1737	// 11:55 pm, hours in high byte, minutes in low byte

#define apptMaxDisplayableAlarms 10

typedef enum alarmTypes {aauMinutes, aauHours, aauDays} AlarmUnitType;

typedef struct {
	SByte					advance;			// Alarm advance (-1 = no alarm)
	AlarmUnitType		advanceUnit;	// minutes, hours, days
} AlarmInfoType;

enum repeatTypes {
	repeatNone, 
	repeatDaily, 
	repeatWeekly, 
	repeatMonthlyByDay, 
	repeatMonthlyByDate,
	repeatYearly
};
typedef enum repeatTypes RepeatType;

typedef struct {
	RepeatType		repeatType;			// daily, weekly, monthlyByDay, etc.
	DateType			repeatEndDate;		// minus one if forever
	unsigned char	repeatFrequency;	// i.e. every 2 days if repeatType daily
	unsigned char	repeatOn;			// monthlyByDay and repeatWeekly only	
	unsigned char  repeatStartOfWeek;// repeatWeekly only
} RepeatInfoType;

typedef RepeatInfoType * RepeatInfoPtr;

typedef struct {
	TimeType			startTime;			// Time the appointment starts
	TimeType			endTime;				// Time the appointment ends
	DateType			date;					// date of appointment
} ApptDateTimeType;

typedef struct {
	UInt    	numExceptions;
	DateType	exception;
} ExceptionsListType;

typedef ExceptionsListType * ExceptionsListPtr;

typedef struct {
	unsigned when			:1;			// set if when info changed (ApptChangeRecord)
	unsigned alarm			:1;			// set if record contains alarm info
	unsigned repeat		:1;			// set if record contains repeat info
	unsigned note			:1;			// set if record contains a note
	unsigned exceptions	:1;			// set if record contains exceptions list
	unsigned description	:1;			
} ApptDBRecordFlags;

typedef struct {
	ApptDateTimeType *	when;
	AlarmInfoType *		alarm;
	RepeatInfoType *		repeat;
	ExceptionsListType *	exceptions;
	CharPtr					description;
	CharPtr					note;
} ApptDBRecordType;

typedef ApptDBRecordType * ApptDBRecordPtr;

typedef struct {
	TimeType		startTime;
	TimeType		endTime;
	UInt			recordNum;
	} ApptInfoType;
typedef ApptInfoType * ApptInfoPtr;

typedef struct {
	ApptDateTimeType 	when;
	ApptDBRecordFlags	flags;	// A flag set for each  datum present
	char				firstField;
} ApptPackedDBRecordType;

typedef ApptPackedDBRecordType * ApptPackedDBRecordPtr;

typedef Int comparF (const void *, const void *, Int other);

extern void DatebookOpenDB(void);
extern void DatebookCloseDB(void);

extern void DatebookCreateRecord(ApptDBRecordPtr r);

#endif
