/* 
 * datedb: utility routines for the creation of summaries and datebook appointments
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: datedb.c,v 1.3 1999/06/03 12:04:31 ga58 Exp $
 *
 * $Log: datedb.c,v $
 * Revision 1.3  1999/06/03 12:04:31  ga58
 *  - Version 3.32
 *
 * Revision 1.2  1999/02/28 16:36:37  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:56  ga58
 *  - Version 3.29
 *
 * Revision 1.7  1999/02/14 17:58:43  ga58
 *  - Version: 3.28
 *
 * Revision 1.6  1999/02/12 17:00:08  ga58
 *  - Version 3.27
 *
 * Revision 1.5  1999/01/31 19:05:03  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.4  1999/01/30 19:08:13  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.3  1999/01/24 20:22:54  ga58
 *  - Beta version 3.23
 *
 * Revision 1.2  1999/01/17 20:02:14  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 * Revision 1.1  1998/12/30 10:27:57  ga58
 * Last changes for the new layout
 *
 * Revision 1.1  1998/12/29 10:10:25  ga58
 * Titrax v3.17 in the new directory
 *
 * Revision 3.9  1998/05/15 16:20:15  ga58
 *  - no visible changes
 *
 * Revision 3.8  1998/05/04 16:25:40  ga58
 *  - changed the summary format to be CSV compatible.
 * CVS:
 * CVS:
 *
 * Revision 3.7  1998/05/03 12:18:30  ga58
 *  - i changed the text format for the summary log to a CSV format.
 * CVS:
 * CVS:
 *
 * Revision 3.6  1998/04/27 06:36:46  ga58
 *  - the width of the command popup window and the project list window are now
 *    calculated dynamically.
 * CVS:
 * CVS:
 *
 * Revision 3.5  1998/04/26 09:03:28  ga58
 *  - finished the conversion to a shared library version
 * CVS:
 * CVS:
 *
 * Revision 3.4  1998/04/19 15:44:26  ga58
 * This is the first branch of the 3.x series of titrax. Most of the new
 * features and code changes are described in the NEWS file.
 * CVS:
 * CVS:
 *
 * Revision 2.7  1998/04/13 09:22:33  ga58
 *  - added a new option: Clear note in datebook mode
 * CVS:
 * CVS:
 *
 * Revision 2.6  1998/04/12 09:31:08  ga58
 *  - a lot of minor changes in the UI
 *  - a lot of minor changes in the code
 *  - added new options
 * CVS:
 * CVS:
 *
 * Revision 2.5  1998/04/10 16:54:09  ga58
 * code cleanups in some modules
 * added the list-code to titraxDB
 * modularized mainForm
 * removed Elapsed and StartedAt from titraxDB and integrated them into
 * titraxPref
 * reworked TextIsEmpty and NoteIsEmpty in titraxDB
 * tried to make the code faster
 * CVS:
 * CVS:
 *
 * Revision 2.4  1998/04/04 15:26:32  ga58
 * version 2.4
 * CVS:
 * CVS:
 *
 * Revision 1.1  1998/04/04 15:24:32  ga58
 * first version in CVS
 * CVS:
 *
 * Revision 1.1.1.1  1998/04/04 15:04:07  ga58
 * first version
 *
 * Revision 2.0  1998/03/24 08:55:56  ga58
 * *** empty log message ***
 *
 * Revision 1.2  1998/03/24 07:50:50  ga58
 * *** empty log message ***
 *
 * Revision 1.1  1998/03/21 16:06:31  ga58
 * Initial revision
 *
 */

#pragma pack(2)

#include "titraxAll.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "datebook.h"
#include "memopad.h"
#include "slotData.h"
#include "slotName.h"
#include "slotNote.h"
#include "project.h"
#include "titraxUtil.h"
#include "titraxPref.h"
#include "datedb.h"

void
AddDatebookAppointment(Int slot, ULong sTime, ULong eTime)
{
  ApptDBRecordType newAppt;
  ApptDateTimeType when;
  DateType date;
  DateTimeType temptime, temptime2;

  DatebookOpenDB();
  
  TimSecondsToDateTime(eTime, &temptime);
  TimSecondsToDateTime(sTime, &temptime2);
  DateSecondsToDate(sTime,&date);
  
  while ((temptime.day != temptime2.day) || (temptime.month != temptime2.month) || (temptime.year != temptime2.year)) {
	 /* We're crossing over a day boundary, make 2 appointments, one on either
		 side of the midnight bondary */
	 when.endTime.hours = 23;
	 when.endTime.minutes = 59;  
	 when.startTime.hours = temptime2.hour;
	 when.startTime.minutes = temptime2.minute;  
	 when.date = date;
	 MemSet (&newAppt, sizeof(newAppt), 0);
	 newAppt.when = &when;
	 newAppt.description = SlotName(slot);
	 newAppt.note = SlotNote(slot);
	 
	 DatebookCreateRecord(&newAppt);

	 sTime = ((sTime/daysInSeconds) + 1)*daysInSeconds;
	 TimSecondsToDateTime(sTime, &temptime2);
	 DateSecondsToDate(sTime,&date);
  }

  when.endTime.hours = temptime.hour;
  when.endTime.minutes = temptime.minute;  
  when.startTime.hours = temptime2.hour;
  when.startTime.minutes = temptime2.minute;  
  when.date = date;
  MemSet (&newAppt, sizeof(newAppt), 0);
  newAppt.when = &when;
  newAppt.description = SlotName(slot);
  newAppt.note = SlotNote(slot);
  
  DatebookCreateRecord(&newAppt);

  DatebookCloseDB();

  SetSlotNote(slot, "");
}

void
AddTimeStamp(Int slot, ULong start, ULong end)
{
  CharPtr tmpP;
  VoidHand tmpH;

  Char timStr[timeStringLength];
  Char dateStr[dateStringLength];
  const Int len = (2 * timeStringLength) + (2 * dateStringLength) + 7;
  Int newLen;

  newLen = StrLen(SlotNote(slot)) + len;
  tmpH = MemHandleNew(newLen + 1);
  tmpP = MemHandleLock(tmpH);

  StrCopy(tmpP, SlotNote(slot));

  if ((StrLen(tmpP) > 0) && (tmpP[StrLen(tmpP) - 1] != '\n'))
	 StrNCat(tmpP, "\n", newLen);

  DateString(dateStr, start);
  TimeString(timStr, start);
  StrNCat(tmpP, dateStr, newLen);
  StrNCat(tmpP, " ", newLen);
  StrNCat(tmpP, timStr, newLen);
  StrNCat(tmpP, "-", newLen);
  DateString(dateStr, end);
  TimeString(timStr, end);
  StrNCat(tmpP, dateStr, newLen);
  StrNCat(tmpP, " ", newLen);
  StrNCat(tmpP, timStr, newLen);
  StrNCat(tmpP, "\n", newLen);

  SetSlotNote(slot, tmpP);

  MemHandleUnlock(tmpH);
  MemHandleFree(tmpH);
}

void
AddAppointment(Int slot, ULong sTime, ULong eTime)
{
  ErrFatalDisplayIf((slot < 0) || (slot >= Prefs()->maxSlot), "AddAppointment->slot");
  ErrFatalDisplayIf(sTime > eTime, "AddAppointment->sTime eTime");

  if (SlotData(slot)->logging == dontLog)
	 return;
  
  if (SlotData(slot)->logging == toDatebook)
	 AddDatebookAppointment(slot, sTime, eTime);
  else
	 AddTimeStamp(slot, sTime, eTime);
}

void
AppendStringToText(CharPtr text, CharPtr string, Int len, Boolean delimiter)
{
  if (Prefs()->summary.quoteFields)
	 StrNCat(text, "\"", len);
  StrNCat(text, string, len);
  if (Prefs()->summary.quoteFields)
	 StrNCat(text, "\"", len);
  if (delimiter) {
	 Char delim[2];
	 delim[0] = delimiterChar[Prefs()->summary.fieldDelimiter];
	 delim[1] = 0;
	 StrNCat(text, delim, len);
  }
}
void
SaveTimeAndNoteInformation(CharPtr text, Int len)
{
  const int minTime = 30; // seconds

  Int i;
  Char timeStr[10];
  ULong sum = SumElapsed();
  DateTimeType dateTime;
  Char dateStr[longDateStrLength];
  Int numOfFields = 0;
  Int fieldsDone;

  if (Prefs()->summary.inclName) numOfFields++;
  if (Prefs()->summary.inclTime) numOfFields++;
  if (Prefs()->summary.inclNote) numOfFields++;

  // build the header line
  if (Prefs()->summary.headerLine) {
	 TimSecondsToDateTime(TimGetSeconds(), &dateTime);
	 DateToAscii(dateTime.month, dateTime.day, dateTime.year, PrefGetPreference(prefDateFormat), dateStr);

	 AppendStringToText(text, dateStr, len, true);
	 AppendStringToText(text, TimeToString(sum, timeStr, 10), len, true);
	 AppendStringToText(text, "days total elapsed time", len, false);
	 StrNCat(text, "\n", len);
  }

  // build the field description
  if (Prefs()->summary.fieldDescr) {
	 fieldsDone = 0;
	 if (Prefs()->summary.inclName) {
		fieldsDone++;
		AppendStringToText(text, "Project Name", len, fieldsDone < numOfFields);
	 }
	 if (Prefs()->summary.inclTime) {
		fieldsDone++;
		AppendStringToText(text, "Elapsed Time", len, fieldsDone < numOfFields);
	 }
	 if (Prefs()->summary.inclNote) {
		fieldsDone++;
		AppendStringToText(text, "Project Note", len, fieldsDone < numOfFields);
	 }
	 if (numOfFields > 0)
		StrNCat(text, "\n", len);
  }

  // build the summary lines
  for(i = 0; i < Prefs()->maxSlot; i++) {
	 if ((SlotIsEmpty(i) == false) && (TotalElapsed(i) > minTime)) {
		fieldsDone = 0;
		if (Prefs()->summary.inclName) {
		  fieldsDone++;
		  AppendStringToText(text, SlotName(i), len, fieldsDone < numOfFields);
		}
		if (Prefs()->summary.inclTime) {
		  fieldsDone++;
		  AppendStringToText(text, TimeToString(TotalElapsed(i), timeStr, 10), len, fieldsDone < numOfFields);
		}
		if (Prefs()->summary.inclNote) {
		  fieldsDone++;
		  AppendStringToText(text, SlotNote(i), len, fieldsDone < numOfFields);
		}
		if (numOfFields > 0)
		  StrNCat(text, "\n", len);
	 }
  }
}

void
CreateDatebookSummary(CharPtr clipText)
{
  ULong sTime = TimGetSeconds();
  Char dateStr[longDateStrLength];
  DateTimeType dateTime;
  Char headerString[80];

  ApptDBRecordType newAppt;
  ApptDateTimeType when;
  DateType date;
  DateTimeType temptime;

  /* build the description */
  TimSecondsToDateTime(TimGetSeconds(), &dateTime);
  DateToAscii(dateTime.month, dateTime.day, dateTime.year, PrefGetPreference(prefDateFormat), dateStr);
  StrCopy(headerString, "Titrax log ");
  StrNCat(headerString, dateStr, 80);

  DatebookOpenDB();
	 
  TimSecondsToDateTime(sTime, &temptime);
  DateSecondsToDate(sTime,&date);

  when.startTime.hours = apptNoTime;
  when.startTime.minutes = apptNoTime;  
  when.endTime.hours = apptNoTime;
  when.endTime.minutes = apptNoTime;
  when.date = date;

  MemSet (&newAppt, sizeof(newAppt), 0);
  newAppt.when = &when;
  newAppt.description = headerString;
  
  newAppt.note = clipText;
  
  DatebookCreateRecord(&newAppt);
	 
  DatebookCloseDB();
}

/*
 * The Save button will stuff all time information as text into a
 * string of maximum length maxLen.
 */
void
CreateMemopadSummary(CharPtr clipText)
{
  Int idx = 0;
  VoidPtr recP;
  VoidHand recH;

  MemopadOpenDB();
  
  (void)MemopadCreateRecordInCategory(&idx, StrLen(clipText) + 1, "Titrax");
  recH = MemopadGetRecordHandle(idx);
  
  recP = MemHandleLock(recH);
  DmStrCopy(recP, 0, clipText);
  MemHandleUnlock(recH);
  
  MemopadReleaseRecordHandle(idx, true);
  
  MemopadCloseDB();
}

void
CreateSummary(void)
{
  VoidHand clip;
  CharPtr clipText;
  
  const Int maxLen = 4096 - 96;

  clip = MemHandleNew(maxLen + 1);
  clipText = (CharPtr)MemHandleLock(clip);
  
  StrCopy(clipText, "");
  
  SaveTimeAndNoteInformation(clipText, maxLen);

  if (Prefs()->saveSummaryTo == memopad)
	 CreateMemopadSummary(clipText);
  else
	 CreateDatebookSummary(clipText);

  MemHandleUnlock(clip);
  MemHandleFree(clip);
}
