/*
 * editForm: the change properties dialog
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: editForm.c,v 1.4 1999/06/23 13:08:14 ga58 Exp $
 *
 * $Log: editForm.c,v $
 * Revision 1.4  1999/06/23 13:08:14  ga58
 * Version 3.34
 *
 * Revision 1.3  1999/06/03 12:04:33  ga58
 *  - Version 3.32
 *
 * Revision 1.2  1999/02/28 16:36:39  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:56  ga58
 *  - Version 3.29
 *
 * Revision 1.4  1999/01/31 19:05:06  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 */

#include "titraxAll.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "callback.h"
#include "slotData.h"
#include "slotName.h"
#include "slotNameList.h"
#include "slotClear.h"
#include "titraxPref.h"
#include "titraxCommands.h"
#include "project.h"
#include "mainForm.h"
#include "editForm.h"

static VoidHand nameH;
static CharPtr nameP;
static FormPtr frm;
static FieldPtr fldP;

static void 
EditFormInit(void)
{
  Int nameLen;

  frm = FrmGetActiveForm();
  fldP = GetObjectPtr(editFormField);

  nameLen = StrLen(SlotName(Prefs()->editedSlot));

  nameH = MemHandleNew(nameLen + 1);
  nameP = MemHandleLock(nameH);

  MemSet(nameP, nameLen + 1, 0);
  StrNCopy(nameP, SlotName(Prefs()->editedSlot), nameLen);

  MemHandleUnlock(nameH);

  FldSetTextHandle(fldP, (Handle)nameH);

  FrmDrawForm(frm);

  FldSetInsPtPosition(fldP, StrLen(SlotName(Prefs()->editedSlot)));
  
  FrmSetControlGroupSelection(frm, 1, editFormDontLog + SlotData(Prefs()->editedSlot)->logging);

  CtlSetValue(GetObjectPtr(editFormIgnore), SlotData(Prefs()->editedSlot)->ignored);

  FrmSetFocus(frm, FrmGetObjectIndex(frm, editFormField));

  if (SlotIsEmpty(Prefs()->editedSlot) == true)
	 GrfSetState(false, false, true);
}

static void
EditFormExit(void)
{
  Boolean newIgnore = CtlGetValue(GetObjectPtr(editFormIgnore));
  loggingType newLog = FrmGetObjectId(frm, FrmGetControlGroupSelection(frm, 1)) - 
	 editFormDontLog;

  FldSetDirty(fldP, true);
  FldSetTextHandle(fldP, 0);
  
  nameP = MemHandleLock(nameH);
  SetSlotName(Prefs()->editedSlot, nameP);
  MemHandleUnlock(nameH);
  MemHandleFree(nameH);
  
  SetSlotData(Prefs()->editedSlot, Logging, &newLog, sizeof(newLog));
  SetSlotData(Prefs()->editedSlot, Ignored, &newIgnore, sizeof(newIgnore));
}

Boolean
EditFormHandleEvent(EventPtr event)
{
  Boolean handled;

  CALLBACK_PROLOGUE

  handled = false;

  switch (event->eType) {
  case ctlSelectEvent:
	 if (event->data.ctlEnter.controlID == editFormOK) {
		EditFormExit();

		if (SlotIsEmpty(Prefs()->editedSlot) == true) {
		  if (ActiveSlot() == Prefs()->editedSlot)
			 Stop(Prefs()->editedSlot);
		  SlotRemove(Prefs()->editedSlot);
		}
		else
		  if (!Prefs()->newUI)
			 Start(Prefs()->editedSlot);

		FrmGotoForm(mainForm);
		handled = true;
	 }
	 break;

  case frmOpenEvent:
	 EditFormInit();
	 Prefs()->actualForm = editForm;
	 handled = true;
	 break;

  case appStopEvent:
	 EditFormExit();
	 handled = false;
	 break;
	 
  case frmTitleSelectEvent:
	 if (Prefs()->menuHack) {
		MenuBarPtr menuP = MenuInit(editFormMenu);

		MenuSetActiveMenu(menuP);
		MenuDrawMenu(menuP);
		handled = true;
	 }
	 else
		handled = false;
	 break;

  case menuEvent:
	 MenuEraseStatus(0);
	 if (event->data.menu.itemID == editFormMenuKeyboard)
		SysKeyboardDialogV10();
	 if (event->data.menu.itemID == editFormMenuGraffiti)
		SysGraffitiReferenceDialog(referenceDefault);
	 if (event->data.menu.itemID == editFormMenuCut)
		FldCut(GetObjectPtr(editFormField));
	 if (event->data.menu.itemID == editFormMenuCopy)
		FldCopy(GetObjectPtr(editFormField));
	 if (event->data.menu.itemID == editFormMenuPaste)
		FldPaste(GetObjectPtr(editFormField));
	 if (event->data.menu.itemID == editFormMenuUndo)
		FldUndo(GetObjectPtr(editFormField));
	 if (event->data.menu.itemID == editFormMenuSelectAll)
		FldSetSelection(GetObjectPtr(editFormField), 0, 
							 FldGetTextLength(GetObjectPtr(editFormField)));
	 handled = true;
	 break;

  default:
	 handled = false;
  }

  CALLBACK_EPILOGUE

  return handled;
}

