/*
 * Form handler for the titrax main form
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: mainForm.c,v 1.5 1999/06/23 13:08:15 ga58 Exp $
 *
 * $Log: mainForm.c,v $
 * Revision 1.5  1999/06/23 13:08:15  ga58
 * Version 3.34
 *
 * Revision 1.4  1999/06/03 12:04:35  ga58
 *  - Version 3.32
 *
 * Revision 1.3  1999/02/28 16:36:40  ga58
 *  - Version 3.31
 *
 * Revision 1.2  1999/02/27 17:39:03  ga58
 *  - Version 3.30
 *
 * Revision 1.1  1999/02/21 13:41:56  ga58
 *  - Version 3.29
 *
 * Revision 1.7  1999/02/14 17:58:44  ga58
 *  - Version: 3.28
 *
 * Revision 1.6  1999/01/31 19:05:07  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.5  1999/01/30 19:08:15  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.4  1999/01/24 20:22:57  ga58
 *  - Beta version 3.23
 *
 * Revision 1.3  1999/01/23 16:02:07  ga58
 *  - checkin before removing all row-constructs
 *
 * Revision 1.2  1999/01/17 20:02:18  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 */

#include "titraxAll.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "callback.h"
#include "titraxPref.h"
#include "titraxUtil.h"
#include "slotName.h"
#include "slotNameList.h"
#include "project.h"
#include "titraxCommands.h"
#include "mainForm.h"

static Int startSlot = 0;
static Int startSlotY = 0;
static Boolean inDnD = false;
static const Int minDnD = 6;
static Boolean inList = false;

static void
MainFormCheckScroll()
{
  SclSetScrollBar(GetObjectPtr(mainFormScrollBar), Prefs()->firstVisibleSlot, 
						 0, Prefs()->maxSlot - VISSLOT, VISSLOT);
}

static void
ScrollTo(Int slot)
{
  if (slot == Prefs()->firstVisibleSlot)
	 return;

  if (slot < 0)
	 slot = 0;

  if (slot > (Prefs()->maxSlot - VISSLOT))
	 slot = Prefs()->maxSlot - VISSLOT;

  Prefs()->firstVisibleSlot = slot;

  UpdateAllSlots();
  MainFormCheckScroll();
}

static void
ScrollUp(Int amount)
{
  ScrollTo(Prefs()->firstVisibleSlot - amount);
}

static void
ScrollDown(Int amount)
{
  ScrollTo(Prefs()->firstVisibleSlot + amount);
}

void
ScrollCenterActive()
{
  if (ActiveSlot() == noSlotActive)
	 return;

  ScrollTo(ActiveSlot() - ((VISSLOT - 1) / 2));
}

static Int
YPositionToSlot(Int y)
{
  Int tableRow;
  RectangleType rect;
  TablePtr tableP = GetObjectPtr(mainFormTable);

  for(tableRow = 0; tableRow < VISSLOT; tableRow++) {
	 TblGetItemBounds(tableP, tableRow, 0, &rect);
	 if (RctPtInRectangle(rect.topLeft.x + 1, y, &rect))
		break;
  }

  return tableRow + Prefs()->firstVisibleSlot;
}

static Int
XPositionToColumn(Int x)
{
  Int column;
  RectangleType rect;
  TablePtr tableP = GetObjectPtr(mainFormTable);

  for(column = 0; column < MAXCOL; column++) {
	 TblGetItemBounds(tableP, 0, column, &rect);
	 if (x < (rect.topLeft.x + rect.extent.x))
		break;
  }

  return column;
}

void
DrawSlot(VoidPtr tableP, Word row, Word column, RectanglePtr bounds)
{
  const Int strLen = 4 * 40;
  Int slot;
  Char text[strLen + 1];
  FontID oldFont;

  CALLBACK_PROLOGUE

  slot = row + Prefs()->firstVisibleSlot;

  TblUnhighlightSelection(tableP);
  WinEraseRectangle(bounds, 0);

  switch (column) {
  case 0: {
	 oldFont = FntSetFont(stdFont);
	 if (!SlotIsEmpty(slot)) {
		GetSlotName(slot, text, strLen);
		TrimToLength(text, stdFont, BUTTONW - 2);
	 }
	 else
		StrNCopy(text, " \205", strLen);
	 break;
  }

  case 1: {
	 oldFont = FntSetFont(stdFont);
	 if (!SlotIsEmpty(slot)) {
		TimeToString(TotalElapsed(slot), text, strLen);
		ExpandLeftToLength(text, strLen, TIMEW-1);
	 }
	 else
		StrNCopy(text, "", strLen);
	 break;
  }

  case 2: {
	 oldFont = FntSetFont(symbolFont);
	 if (!SlotIsEmpty(slot)) {
		if (NoteIsEmpty(slot))
		  StrNCopy(text, "\013", strLen);
		else
		  StrNCopy(text, "\014", strLen);
	 }
	 else
		StrNCopy(text, "", strLen);
	 break;
  }

  default:
	 return;
  }
  
  WinDrawChars(text, StrLen(text), bounds->topLeft.x, bounds->topLeft.y);
  FntSetFont(oldFont);

  if ((ActiveSlot() == slot) && ((column == 0) || (column == 1)))
	 WinInvertRectangle(bounds, 0);
  
  CALLBACK_EPILOGUE
}

static void
MainFormInit(void)
{
  Int tableRow;
  Int column;
  FormPtr frm = FrmGetActiveForm();
  TablePtr tableP = GetObjectPtr(mainFormTable);
  RectangleType rect1 = {{0, 139}, {160, 2}};

  for(tableRow = 0; tableRow < VISSLOT; tableRow++) {
	 TblSetRowUsable(tableP, tableRow, true);

	 for(column = 0; column < MAXCOL; column++)
		TblSetItemStyle(tableP, tableRow, column, customTableItem);
  }
  
  for(column = 0; column < MAXCOL; column++) {
	 TblSetCustomDrawProcedure(tableP, column, DrawSlot);
	 TblSetColumnUsable(tableP, column, true);
  }

  TblSetColumnSpacing(tableP, 0, 0);
  TblSetColumnSpacing(tableP, 1, SPACE);
  TblSetColumnSpacing(tableP, 2, 0);

  SlotNameListUpdate();
  UpdateProjectListButton();

  CtlSetValue(GetObjectPtr(mainFormNewUI), Prefs()->newUI);

  FrmDrawForm(frm);

  MainFormCheckScroll();
  UpdateSumTimeButton();

  WinDrawRectangle(&rect1, 0);
}

Boolean
MainFormHandleEvent(EventPtr event)
{
  Boolean handled = false;
  TablePtr tableP = GetObjectPtr(mainFormTable);
  RectangleType rect;

  CALLBACK_PROLOGUE

  switch (event->eType) {
  case ctlSelectEvent: {
	 if (event->data.ctlSelect.controlID == mainFormCommand) {
		ListPtr listP;
		inList = true;

		if (ActiveSlot() != noSlotActive)
		  listP = GetObjectPtr(mainFormCommandList1);
		else
		  listP = GetObjectPtr(mainFormCommandList2);

		LstSetSelection(listP, noListSelection);
		LstSetHeight(listP, LstGetNumberOfItems(listP));
		LstSetPosition(listP, 3, 155 - (listP->bounds.extent.y));

		DoCommand(LstPopupList(listP), ActiveSlot());
		
		inList = false;
		handled = true;
	 }
		
	 if (event->data.ctlSelect.controlID == mainFormProjectPopup)
		handled = (SlotNameListNumOfEntries() == 0);

	 if (event->data.ctlSelect.controlID == mainFormNewUI) {
		Prefs()->newUI = !Prefs()->newUI;
		CtlSetValue(GetObjectPtr(mainFormNewUI), Prefs()->newUI);
		handled = true;
	 }
	 break;
  }

  case penDownEvent: {
	 if (inList) {
		inDnD = false;
		handled = false;
		break;
	 }
	 TblGetBounds(tableP, &rect);
	 if (RctPtInRectangle(event->screenX, event->screenY, &rect)) {
		startSlotY = event->screenY;
		startSlot = YPositionToSlot(event->screenY);
		inDnD = true;
		handled = true;
	 }
	 else {
		inDnD = false;
		handled = false;
	 }
	 break;
  }

  case penUpEvent: {
	 if (inDnD) {
		inDnD = false;
		TblGetBounds(tableP, &rect);
		if (RctPtInRectangle(event->screenX, event->screenY, &rect)) {
		  Int endSlot = YPositionToSlot(event->screenY);
		  if (startSlot != endSlot) {
			 if (abs(startSlotY - event->screenY) >= minDnD) {
				SwapSlots(startSlot, endSlot);
				SlotNameListUpdate();
				UpdateProjectListButton();
				UpdateSlot(startSlot);
				UpdateSlot(endSlot);
			 }
			 else
				UpdateSlot(endSlot);
		  }
		  else {
			 Int column = XPositionToColumn(event->screenX);
			 Int slot = endSlot;
				
			 if (column < 2) {
				if (!Prefs()->newUI) {
				  if (ActiveSlot() == slot)
					 StopAndUpdate(slot);
				  else {
					 if (SlotIsEmpty(slot))
						DoCommand(changeName, slot); // Change Name
					 if (!SlotIsEmpty(slot))
						StartAndUpdate(slot);
				  }
				}
				else { /* NewUI */
				  ListPtr listP;

				  if (SlotIsEmpty(slot))
					 listP = GetObjectPtr(mainFormProjList3);
				  else {
					 if (slot != ActiveSlot())
						listP = GetObjectPtr(mainFormProjList1);
					 else
						listP = GetObjectPtr(mainFormProjList2);
				  }

				  LstSetHeight(listP, LstGetNumberOfItems(listP));
				  LstSetSelection(listP, noListSelection);
				  LstSetPosition(listP, event->screenX - 4, event->screenY - 4);

				  DoCommand(LstPopupList(listP) + startStop, slot);

				  LstSetSelection(listP, noListSelection);
				}
			 }
			 
			 if (column == 2) {
				if (!SlotIsEmpty(slot))
				  DoCommand(editNote, slot);
				else
				  UpdateSlot(slot);
			 }
		  }
		}
		handled = true;
	 }
	 else
		handled = false;

	 break;
  }

  case penMoveEvent: {
	 if (inDnD) {
		TblGetBounds(tableP, &rect);
		if (!RctPtInRectangle(event->screenX, event->screenY, &rect)) {
		  Int xD, yD;
		  Boolean penD;
		  
		  TblUnhighlightSelection(tableP);
		  
		  if (event->screenY < rect.topLeft.y)
			 do {
				ScrollUp(1);
				EvtGetPen(&xD, &yD, &penD);
			 } while (penD && (yD < rect.topLeft.y));
		  if (event->screenY > rect.topLeft.y + rect.extent.y)
			 do {
				ScrollDown(1);
				EvtGetPen(&xD, &yD, &penD);
			 } while (penD && (yD > rect.topLeft.y + rect.extent.y));
		}
		else {
		  Int currentSlot = YPositionToSlot(event->screenY);
		  if (abs(startSlotY - event->screenY) >= minDnD)
			 TblSelectItem(tableP, currentSlot - Prefs()->firstVisibleSlot, 0);
		}
		handled = true;
	 }
	 else
		handled = false;

	 break;
  }

  case popSelectEvent:
	 if (event->data.popSelect.listID == mainFormProjectList) {
		Word selection = event->data.popSelect.selection;
	 
		if (selection != noListSelection) {
		  Int slotNo = SlotNameListNumToSlot(selection);
		  if (ActiveSlot() == slotNo)
			 StopAndUpdate(slotNo);
		  else
			 StartAndUpdate(slotNo);

		  ScrollCenterActive();
		}
		handled = true; // we redraw the popuptrigger
	 }
	 break;

  case frmOpenEvent:
	 MainFormInit();
	 Prefs()->actualForm = mainForm;
	 handled = true;
	 break;

  case frmCloseEvent:
	 inDnD = false;
	 inList = false;
	 handled = false;
	 break;

  case nilEvent:
	 if (FrmGetActiveFormID() == mainForm) {
		UpdateActiveSlot();
		UpdateSumTimeButton();
	 }
	 handled = false;
	 break;

  case sclRepeatEvent:
	 ScrollTo(event->data.sclRepeat.newValue);
	 handled = false;
	 break;

  case keyDownEvent:
	 if (event->data.keyDown.chr == pageUpChr) {
		ScrollUp(VISSLOT);
		handled = true;
	 }

	 if (event->data.keyDown.chr == pageDownChr) {
		ScrollDown(VISSLOT);
		handled = true;
	 }
	 break;

  case frmTitleSelectEvent:
	 if (Prefs()->menuHack) {
		MenuBarPtr menuP = MenuInit(mainFormMenu);

		MenuSetActiveMenu(menuP);
		MenuDrawMenu(menuP);
		handled = true;
	 }
	 else
		handled = false;
	 break;

  case menuEvent:
	 MenuEraseStatus(0);
	 if (event->data.menu.itemID == mainFormMenuOption) {
		FrmGotoForm(optionForm);
		handled = true;
	 }
	 if (event->data.menu.itemID == mainFormMenuAbout) {
		FormPtr frm1 = FrmInitForm(aboutForm);
		FormPtr frm2 = FrmInitForm(aboutForm2);

		FrmDoDialog(frm1);
		FrmDoDialog(frm2);
		FrmDeleteForm(frm2);
		FrmDeleteForm(frm1);

		handled = true;
	 }
	 break;

  default:
	 handled = false;
  }

  CALLBACK_EPILOGUE

  return handled;
}

