/*
 * memopad.c
 *
 * This module provides interface functions for the memopad
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: memopad.c,v 1.3 1999/06/03 12:04:37 ga58 Exp $
 *
 * $Log: memopad.c,v $
 * Revision 1.3  1999/06/03 12:04:37  ga58
 *  - Version 3.32
 *
 * Revision 1.2  1999/02/27 17:39:04  ga58
 *  - Version 3.30
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.1  1999/01/17 20:02:19  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 */

#pragma pack(2)

#include "titraxAll.h"
#include "titraxID.h"
#include "memopad.h"

static DmOpenRef MemopadDB = 0;
static UInt usageCount = 0;

void
MemopadOpenDB(void)
{
  usageCount++;

  if (MemopadDB == 0) {
	 MemopadDB = DmOpenDatabaseByTypeCreator('DATA', 'memo', dmModeReadWrite);
	 if (MemopadDB == 0)
		ErrDisplay("Cannot open memopad database.");
  }
}

void 
MemopadCloseDB(void)
{
  usageCount--;

  if (usageCount == 0) {
	 DmCloseDatabase(MemopadDB);
	 MemopadDB = 0;
  }
}

Word
MemopadCreateCategory(CharPtr categoryName)
{
  Int i;
  Word category;
  Char categoryStr[dmCategoryLength + 1];

  category = CategoryFind(MemopadDB, categoryStr);
  if ((category != dmAllCategories) && (category != dmUnfiledCategory))
	 return category;

  for(i = 0; i < 16; i++) {
	 CategoryGetName(MemopadDB, i, categoryStr);
	 if (StrLen(categoryStr) == 0)
		break;
	 else
		if (i == 15) {
		  (void)FrmAlert(titraxNoCategoryAlert);
		  return dmUnfiledCategory;
		}
  }
  CategorySetName(MemopadDB, i, categoryName);

  return CategoryFind(MemopadDB, categoryName);
}

void
MemopadSetRecordToCategory(UInt idx, CharPtr categoryName)
{
  Word attr;
  Word category;
  Char categoryStr[dmCategoryLength + 1];


  DmRecordInfo(MemopadDB, idx, &attr, NULL, NULL);

  attr &= ~dmRecAttrCategoryMask;

  category = CategoryFind(MemopadDB, categoryName);

  if ((category == dmAllCategories) || (category == dmUnfiledCategory)) {
	 StrNCopy(categoryStr, categoryName, dmCategoryLength);
	 category = MemopadCreateCategory(categoryStr);
  }

  attr |= category;

  DmSetRecordInfo(MemopadDB, idx, &attr, NULL);
}

Boolean
MemopadUidxToIdx(ULong uidx, Int* idxP)
{
  UInt attr;

  if (DmFindRecordByID(MemopadDB, uidx, idxP) != 0)
	 return false;

  if (DmRecordInfo(MemopadDB, *idxP, &attr, NULL, NULL) != 0)
	 return false;

  if (attr & dmRecAttrDelete)
	 return false;

  return true;
}

ULong
MemopadCreateRecordInCategory(Int* idxP, Int size, CharPtr categoryName)
{
  ULong uidx;
  CharPtr recP;
  VoidHand recH;

  *idxP = 0;

  recH = DmNewRecord(MemopadDB, idxP, size);

  recP = MemHandleLock(recH);
  DmSet(recP, 0, size, 0);
  MemHandleUnlock(recH);

  DmReleaseRecord(MemopadDB, *idxP, true);

  MemopadSetRecordToCategory(*idxP, categoryName);

  DmRecordInfo(MemopadDB, *idxP, NULL, &uidx, NULL);

  return uidx;
}

VoidHand
MemopadGetRecordHandle(Int idx)
{
  return DmGetRecord(MemopadDB, idx);
}

VoidHand
MemopadQueryRecordHandle(Int idx)
{
  return DmQueryRecord(MemopadDB, idx);
}

VoidHand
MemopadResizeRecordHandle(Int idx, Int newSize)
{
  return DmResizeRecord(MemopadDB, idx, newSize);
}

void
MemopadReleaseRecordHandle(Int idx, Boolean dirty)
{
  DmReleaseRecord(MemopadDB, idx, dirty);
}

void
MemopadRemoveRecord(Int idx)
{
  DmRemoveRecord(MemopadDB, idx);
}
