/*
 * noteForm.c: eventhandler for the edit-note form
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: noteForm.c,v 1.3 1999/06/03 12:04:38 ga58 Exp $
 *
 * $Log: noteForm.c,v $
 * Revision 1.3  1999/06/03 12:04:38  ga58
 *  - Version 3.32
 *
 * Revision 1.2  1999/02/28 16:36:41  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.4  1999/01/31 19:05:08  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.3  1999/01/24 20:22:58  ga58
 *  - Beta version 3.23
 *
 * Revision 1.2  1999/01/17 20:02:20  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 */

#include "titraxAll.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "callback.h"
#include "slotData.h"
#include "slotName.h"
#include "slotNote.h"
#include "titraxPref.h"
#include "titraxUtil.h"
#include "mainForm.h"
#include "project.h"
#include "noteForm.h"

static FormPtr frm;
static FieldPtr fldP;

static void
NoteFormCheckScroll()
{
  Boolean canUp, canDown;

  frm = FrmGetActiveForm();
  canUp = FldScrollable(GetObjectPtr(noteFormField), up);
  canDown = FldScrollable(GetObjectPtr(noteFormField), down);
  
  FrmUpdateScrollers(frm, 
							FrmGetObjectIndex(frm, noteFormUp), 
							FrmGetObjectIndex(frm, noteFormDown),
							canUp, canDown);
}

static void
NoteFormInit(void)
{
  const Int titleLen = 40;
  Char title[titleLen + 1];
  Int noteLen;

  noteLen = StrLen(SlotNote(Prefs()->editedSlot));
  frm = FrmGetActiveForm();
  fldP = GetObjectPtr(noteFormField);

  FrmDrawForm(frm);

  StrNCopy(title, "Note for ", titleLen);
  StrNCat(title, SlotName(Prefs()->editedSlot), titleLen);
  TrimToLength(title, boldFont, 155);

  FrmCopyTitle(frm, title);

  FldSetTextHandle(fldP, SlotNoteHandleGet(Prefs()->editedSlot));
  FldSetInsPtPosition(fldP, noteLen);

  FrmSetFocus(frm, FrmGetObjectIndex(frm, noteFormField));

  if (NoteIsEmpty(Prefs()->editedSlot) == true)
	 GrfSetState(false, false, true);

  FldDrawField(GetObjectPtr(noteFormField));

  NoteFormCheckScroll();
}

static void
NoteFormExit(void)
{
  FldCompactText(fldP);
  FldSetDirty(GetObjectPtr(noteFormField), true);
  FldSetTextHandle(GetObjectPtr(noteFormField), 0);
  SlotNoteHandleRelease(Prefs()->editedSlot);

}

static void
ScrollField(Int amount, Int direction)
{
  if (FldScrollable(fldP, direction))
	 FldScrollField(fldP, amount, direction);

  NoteFormCheckScroll();
}

Boolean
NoteFormHandleEvent(EventPtr event)
{
  Boolean handled = false;

  CALLBACK_PROLOGUE

  switch (event->eType) {
  case ctlSelectEvent:
	 if (event->data.ctlEnter.controlID == noteFormOk) {
		NoteFormExit();

		FrmGotoForm(mainForm);

		handled = true;
	 }

	 if (event->data.ctlEnter.controlID == noteFormTimeStamp) {
		Char timStr[timeStringLength];

		TimeString(timStr, TimGetSeconds());
		FldInsert(fldP, timStr, StrLen(timStr));
		NoteFormCheckScroll();
		handled = true;
	 }

	 if (event->data.ctlEnter.controlID == noteFormDateStamp) {
		Char datStr[dateStringLength];

		DateString(datStr, TimGetSeconds());
		FldInsert(fldP, datStr, StrLen(datStr));
		NoteFormCheckScroll();
		handled = true;
	 }
	 break;

  case ctlRepeatEvent:
	 if (event->data.ctlRepeat.controlID == noteFormUp) {
		ScrollField(1, up);
		handled = false;
	 }
	 else if (event->data.ctlRepeat.controlID == noteFormDown) {
		ScrollField(1, down);
		handled = false;
	 }
	 break;

  case keyDownEvent:
	 if (event->data.keyDown.chr == pageUpChr) {
		ScrollField(FldGetVisibleLines(GetObjectPtr(noteFormField)) - 1, up);
		handled = true;
	 }
	 else if (event->data.keyDown.chr == pageDownChr) {
		ScrollField(FldGetVisibleLines(GetObjectPtr(noteFormField)) - 1, down);
		handled = true;
	 }
	 else {
		handled = FrmHandleEvent(FrmGetActiveForm(), event);
		NoteFormCheckScroll();
	 }
	 break;

  case frmOpenEvent:
	 NoteFormInit();
	 Prefs()->actualForm = noteForm;
	 handled = true;
	 break;

  case appStopEvent:
	 NoteFormExit();
	 handled = false;
	 break;

  case frmTitleSelectEvent:
	 if (Prefs()->menuHack) {
		MenuBarPtr menuP = MenuInit(noteFormMenu);

		MenuSetActiveMenu(menuP);
		MenuDrawMenu(menuP);
		handled = true;
	 }
	 else
		handled = false;
	 break;

  case menuEvent:
	 MenuEraseStatus(0);
	 if (event->data.menu.itemID == noteFormMenuKeyboard)
		SysKeyboardDialogV10();
	 if (event->data.menu.itemID == noteFormMenuGraffiti)
		SysGraffitiReferenceDialog(referenceDefault);
	 if (event->data.menu.itemID == noteFormMenuCut)
		FldCut(GetObjectPtr(noteFormField));
	 if (event->data.menu.itemID == noteFormMenuCopy)
		FldCopy(GetObjectPtr(noteFormField));
	 if (event->data.menu.itemID == noteFormMenuPaste)
		FldPaste(GetObjectPtr(noteFormField));
	 if (event->data.menu.itemID == noteFormMenuUndo)
		FldUndo(GetObjectPtr(noteFormField));
	 if (event->data.menu.itemID == noteFormMenuSelectAll)
		FldSetSelection(GetObjectPtr(noteFormField), 0, 
							 FldGetTextLength(GetObjectPtr(noteFormField)));
	 NoteFormCheckScroll();

	 handled = true;
	 break;

  default:
	 handled = false;
  }

  CALLBACK_EPILOGUE

  return handled;
}

