/*
 * optionForm: settings dialog
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: optionForm.c,v 1.3 1999/06/03 12:04:40 ga58 Exp $
 *
 * $Log: optionForm.c,v $
 * Revision 1.3  1999/06/03 12:04:40  ga58
 *  - Version 3.32
 *
 * Revision 1.2  1999/02/28 16:36:42  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.5  1999/02/04 16:16:13  ga58
 *  - Version 3.26, new acceleration algorithm
 *
 * Revision 1.4  1999/01/31 19:05:09  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 */

#include "titraxAll.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "callback.h"
#include "project.h"
#include "titraxPref.h"
#include "titraxAllDBs.h"
#include "mainForm.h"
#include "titraxUtil.h"
#include "accelerate.h"
#include "optionForm.h"

#define numSlotsLen 10
static Char numSlots[numSlotsLen + 1];

static Int newSize;

static FormPtr frm;

static void 
OptionFormCheckScroll(void)
{
  FrmUpdateScrollers(frm,
							FrmGetObjectIndex(frm, optionFormUp),
							FrmGetObjectIndex(frm, optionFormDown), 
							newSize < MAXSLOT + 1, newSize > VISSLOT);
}

static void
OptionFormInit(void)
{
  frm = FrmGetActiveForm();

  FrmDrawForm(frm);

  StrNCopy(numSlots, StrIToA(numSlots, Prefs()->maxSlot), numSlotsLen);
  ExpandLeftToLength(numSlots, numSlotsLen, 25);
  CtlSetLabel(GetObjectPtr(optionFormSlots), numSlots);

  newSize = Prefs()->maxSlot;

  CtlSetValue(GetObjectPtr(optionFormDays), Prefs()->timeFormat == days);
  CtlSetValue(GetObjectPtr(optionFormNoDays), Prefs()->timeFormat == noDays);

  CtlSetValue(GetObjectPtr(optionFormDatebook), Prefs()->saveSummaryTo == datebook);
  CtlSetValue(GetObjectPtr(optionFormMemopad), Prefs()->saveSummaryTo == memopad);

  CtlSetValue(GetObjectPtr(optionFormWarnings), Prefs()->shiftWarnings);

  CtlSetValue(GetObjectPtr(optionFormMenuHack), Prefs()->menuHack);

  OptionFormCheckScroll();

  Accel_Init(1);
}

Boolean
OptionFormHandleEvent(EventPtr event)
{
  Boolean handled;

  CALLBACK_PROLOGUE

  handled = false;

  Accel_Handle(event->eType == ctlRepeatEvent);

  switch (event->eType) {

  case ctlSelectEvent:
	 if (event->data.ctlEnter.controlID == optionFormOk) {
		Prefs()->shiftWarnings = CtlGetValue(GetObjectPtr(optionFormWarnings));
		Prefs()->menuHack = CtlGetValue(GetObjectPtr(optionFormMenuHack));

		if (CtlGetValue(GetObjectPtr(optionFormDatebook)))
		  Prefs()->saveSummaryTo = datebook;
		else
		  Prefs()->saveSummaryTo = memopad;

		if (CtlGetValue(GetObjectPtr(optionFormDays)))
		  Prefs()->timeFormat = days;
		else
		  Prefs()->timeFormat = noDays;

		/*
		 * if the user has reduced the number of slots, check whether we
		 * would remove slots, which have been used and give the user a
		 * chance to quit.
		 */
		if (newSize < Prefs()->maxSlot) {
		  Int i;
		  Int lost = 0;
		  
		  for(i = newSize; i < Prefs()->maxSlot; i++)
			 if (!SlotIsEmpty(i))
				lost++;

		  if (lost != 0)
			 if (FrmAlert(titraxResizeAlert) == 0)
				newSize = Prefs()->maxSlot;
		}

		if (ActiveSlot() >= newSize)
		  Stop(ActiveSlot());

		if ((Prefs()->firstVisibleSlot + VISSLOT) > newSize)
		  Prefs()->firstVisibleSlot = 0;

		ResizeAllDBs(newSize);

		FrmGotoForm(mainForm);

		handled = true;
	 }
	 break;

  case ctlRepeatEvent: {
	 Int incr = Accel_Incr();

	 if (event->data.ctlRepeat.controlID == optionFormUp)
		newSize += incr;
	 if (event->data.ctlRepeat.controlID == optionFormDown)
		newSize -= incr;

	 if (newSize < VISSLOT)
		newSize = VISSLOT;

	 if (newSize > MAXSLOT)
		newSize = MAXSLOT;

	 StrNCopy(numSlots, StrIToA(numSlots, newSize), numSlotsLen);
	 ExpandLeftToLength(numSlots, numSlotsLen, 25);
	 CtlSetLabel(GetObjectPtr(optionFormSlots), numSlots);
	 
	 OptionFormCheckScroll();

	 handled = false;
	 break;
  }

  case frmOpenEvent:
	 OptionFormInit();
	 Prefs()->actualForm = optionForm;
	 handled = true;
	 break;

  default:
	 handled = false;
  }

  CALLBACK_EPILOGUE

  return handled;
}
