/*
 * shiftForm: shift time dialog handling
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: shiftForm.c,v 1.3 1999/06/03 12:04:45 ga58 Exp $
 *
 * $Log: shiftForm.c,v $
 * Revision 1.3  1999/06/03 12:04:45  ga58
 *  - Version 3.32
 *
 * Revision 1.2  1999/02/28 16:36:44  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.6  1999/02/04 16:16:14  ga58
 *  - Version 3.26, new acceleration algorithm
 *
 * Revision 1.5  1999/01/31 19:05:11  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 */

#include "titraxAll.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "callback.h"
#include "slotData.h"
#include "slotName.h"
#include "slotNameList.h"
#include "titraxPref.h"
#include "titraxUtil.h"
#include "drawDigits.h"
#include "project.h"
#include "mainForm.h"
#include "accelerate.h"
#include "shiftForm.h"

static Int selectedShiftSlot;
static Long fromElapsed;
static Long fromOrigElapsed;
static Long toElapsed;
static Long toOrigElapsed;
static Long minAmount;
static Long maxAmount;
static Long toMinVal;
static Long toMaxVal;

#define nameLen 40
static Char fromName[nameLen + 1];
static Char toName[nameLen + 1];

static FormPtr frm;

static void
ShiftFormCheckScroll(Boolean canUp, Boolean canDown)
{
  FrmUpdateScrollers(frm, 
							FrmGetObjectIndex(frm, shiftFormUp),
							FrmGetObjectIndex(frm, shiftFormDown), 
							canUp, canDown);
}

static void
DrawTimes(void)
{
  DrawDigits(fromElapsed, shiftFormTimeFrom, 45, stdFont);
  DrawDigits(toElapsed, shiftFormTimeTo, 45, stdFont);
	 
  if (selectedShiftSlot != Prefs()->editedSlot)
	 ShiftFormCheckScroll(toElapsed > toMinVal, toElapsed < toMaxVal);
  else
	 ShiftFormCheckScroll(false, false);
}

static void 
HandleSlot(Int slot, CharPtr name, Word id, Long* origElapsed, Long* elapsed)
{
  GetSlotName(slot, name, nameLen);
  TrimToLength(name, stdFont, 95 - 12);
  CtlSetLabel(GetObjectPtr(id), name);
  
  *origElapsed = TotalElapsed(slot);
  *elapsed = *origElapsed;
}

static void
Calculate(void)
{
  CalculateMinMaxShiftAmount(Prefs()->editedSlot, selectedShiftSlot, &minAmount, &maxAmount);
  
  toMinVal = toOrigElapsed + minAmount;
  toMaxVal = toOrigElapsed + maxAmount;
}

static void
ShiftFormInit(void)
{
  Int i;
  ListPtr listP;
  
  frm = FrmGetActiveForm();

  if (SlotNameListNumOfEntries() < 2) {
	 (void)FrmAlert(titraxShiftNoProjectsAlert);
	 FrmGotoForm(mainForm);
  }

  FrmDrawForm(frm);

  // first slot
  HandleSlot(Prefs()->editedSlot, fromName, shiftFormPopup2, &fromOrigElapsed, &fromElapsed);

  // second slot
  listP = GetObjectPtr(shiftFormList);

  LstSetListChoices(listP, SlotNameListP(), SlotNameListNumOfEntries());
  LstSetHeight(listP, SlotNameListNumOfEntries());
  
  for(i = 0; i < SlotNameListNumOfEntries(); i++)
	 if (SlotNameListNumToSlot(i) != Prefs()->editedSlot) {
		LstSetSelection(listP, i);

		selectedShiftSlot = SlotNameListNumToSlot(i);
		break;
	 }
  
  HandleSlot(selectedShiftSlot, toName, shiftFormPopup, &toOrigElapsed, &toElapsed);
  
  Calculate();

  DrawTimes();

  Accel_Init(60);
}

Boolean
ShiftFormHandleEvent(EventPtr event)
{
  Boolean handled;

  CALLBACK_PROLOGUE

  handled = false;

  Accel_Handle(event->eType == ctlRepeatEvent);

  switch (event->eType) {
  case ctlSelectEvent:
	 if (event->data.ctlEnter.controlID == shiftFormCommit) {
		ShiftTime(Prefs()->editedSlot, selectedShiftSlot, toElapsed - toOrigElapsed);

	   FrmGotoForm(mainForm);
	   handled = true;
	 }

	 if (event->data.ctlEnter.controlID == shiftFormCancel) {
		FrmGotoForm(mainForm);
		handled = true;
	 }
	 break;

  case ctlRepeatEvent: {
	 Int incr = Accel_Incr();

	 if (event->data.ctlEnter.controlID == shiftFormUp)
		toElapsed -= incr;
	 if (event->data.ctlEnter.controlID == shiftFormDown)
		toElapsed += incr;
	 
	 if (toElapsed < toMinVal)
		toElapsed = toMinVal;

	 if (toElapsed > toMaxVal)
		toElapsed = toMaxVal;
	 
	 fromElapsed = fromOrigElapsed - (toElapsed - toOrigElapsed);
	 
	 DrawTimes();
	 handled = false;
	 break;
  }

  case popSelectEvent: {
	 Word selection = event->data.popSelect.selection;

	 fromOrigElapsed = TotalElapsed(Prefs()->editedSlot);
	 fromElapsed = fromOrigElapsed;

	 selectedShiftSlot = SlotNameListNumToSlot(selection);

	 HandleSlot(selectedShiftSlot, toName, shiftFormPopup, &toOrigElapsed, &toElapsed);
	 
	 Calculate();
	 
	 DrawTimes();
	 handled = true; // we redraw the button, not the OS!
	 break;
  }

  case frmOpenEvent: {
	 ShiftFormInit();
	 Prefs()->actualForm = shiftForm;
	 handled = true;
	 break;
  }

  default:
	 handled = false;
  }

  CALLBACK_EPILOGUE

  return handled;
}
