/*
 * slotData.c
 *
 * Titrax database subroutines for project data
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: slotData.c,v 1.3 1999/06/03 12:04:48 ga58 Exp $
 *
 * $Log: slotData.c,v $
 * Revision 1.3  1999/06/03 12:04:48  ga58
 *  - Version 3.32
 *
 * Revision 1.2  1999/02/28 16:36:47  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.3  1999/01/31 19:05:16  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.2  1999/01/30 19:08:21  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.1  1999/01/24 20:23:05  ga58
 *  - Beta version 3.23
 *
 */

#pragma pack(2)

#include "titraxAll.h"
#include "titraxConst.h"
#include "titraxPref.h"
#include "slotDBUtil.h"
#include "slotData.h"

static DmOpenRef TitraxDB;

#define titraxDBType 'Data'
#define titraxDBName "TitraxDataDB"
#define titraxDBVersion 4

VoidHand slotDataHandle;
slotDataType** slotData;

static VoidHand slotHandHandle;
static VoidHand* slotHand;

static ULong offset[5];

/*
 * Database operations
 * 
 * These routines do NOT perform any error-checking
 */
void
OpenSlotDataDatabase(void)
{
  slotHandHandle = MemHandleNew(sizeof(VoidHand[Prefs()->maxSlot]));
  slotHand = MemHandleLock(slotHandHandle);

  slotDataHandle = MemHandleNew(sizeof(VoidPtr[Prefs()->maxSlot]));
  slotData = MemHandleLock(slotDataHandle);

  OpenDatabase(&TitraxDB, titraxDBName, titraxDBType, titraxAppType, slotHand, (VoidPtr*)slotData, sizeof(slotDataType), titraxDBVersion);

  offset[Elapsed] = (void*)&(slotData[0]->elapsed) - (void*)slotData[0];
  offset[StartedAt] = (void*)&(slotData[0]->startedAt) - (void*)slotData[0];
  offset[Active] = (void*)&(slotData[0]->active) - (void*)slotData[0];
  offset[Logging] = (void*)&(slotData[0]->logging) - (void*)slotData[0];
  offset[Ignored] = (void*)&(slotData[0]->ignored) - (void*)slotData[0];
}

void
CloseSlotDataDatabase(void)
{
  CloseDatabase(TitraxDB, slotHand);

  MemHandleUnlock(slotDataHandle);
  MemHandleFree(slotDataHandle);

  MemHandleUnlock(slotHandHandle);
  MemHandleFree(slotHandHandle);
}

slotDataType*
SlotData(Int slot)
{
  return slotData[slot];
}

void
SetSlotData(Int slot, offsetType what, VoidPtr data, Int size)
{
  DmWrite(slotData[slot], offset[what], data, size);
}

void
SwapSlotData(Int slot1, Int slot2)
{
  slotDataType tmp;

  MemMove(&tmp, slotData[slot1], sizeof(slotDataType));

  DmWrite(slotData[slot1], 0, slotData[slot2], sizeof(slotDataType));
  DmWrite(slotData[slot2], 0, &tmp, sizeof(slotDataType));
}
