/*
 * slotNameList: utility routines for the handling of a list of all non-empty slots
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: slotNameList.c,v 1.2 1999/02/28 16:36:50 ga58 Exp $
 *
 * $Log: slotNameList.c,v $
 * Revision 1.2  1999/02/28 16:36:50  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.2  1999/01/30 19:08:25  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.1  1999/01/24 20:23:05  ga58
 *  - Beta version 3.23
 *
 */

#include "titraxAll.h"
#include "titraxConst.h"
#include "slotData.h"
#include "slotName.h"
#include "project.h"
#include "titraxPref.h"
#include "slotNameList.h"

typedef struct {
  CharPtr* listP;
  Int* listToSlot;
  Int* slotToList;
  Int entries;
} slotNameListType;

VoidHand listPHandle;
VoidHand listToSlotHandle;
VoidHand slotToListHandle;

static VoidHand slotNameListHand;
static slotNameListType* slotNameList;

void
OpenSlotNameList(void)
{
  slotNameListHand = MemHandleNew(sizeof(slotNameListType));
  if (slotNameListHand == 0)
	 ErrDisplay("SlotNameListCreate: MemHandleNew failed.");

  slotNameList = (slotNameListType*)MemHandleLock(slotNameListHand);

  listPHandle = MemHandleNew(sizeof(CharPtr[Prefs()->maxSlot]));
  slotNameList->listP = MemHandleLock(listPHandle);
  
  listToSlotHandle = MemHandleNew(sizeof(Int[Prefs()->maxSlot]));
  slotNameList->listToSlot = MemHandleLock(listToSlotHandle);

  slotToListHandle = MemHandleNew(sizeof(Int[Prefs()->maxSlot]));
  slotNameList->slotToList = MemHandleLock(slotToListHandle);

  SlotNameListUpdate();
}

void
CloseSlotNameList(void)
{
  MemHandleUnlock(slotToListHandle);
  MemHandleFree(slotToListHandle);

  MemHandleUnlock(listToSlotHandle);
  MemHandleFree(listToSlotHandle);

  MemHandleUnlock(listPHandle);
  MemHandleFree(listPHandle);

  MemHandleUnlock(slotNameListHand);
  MemHandleFree(slotNameListHand);
}

void
SlotNameListUpdate(void)
{
  Int slot;

  slotNameList->entries = 0;

  for(slot = 0; slot < Prefs()->maxSlot; slot++) {
	 if (!SlotIsEmpty(slot)) {
		slotNameList->listP[slotNameList->entries] = SlotName(slot);
		slotNameList->listToSlot[slotNameList->entries] = slot;
		slotNameList->slotToList[slot] = slotNameList->entries;
		slotNameList->entries++;
	 }
  }
}

Int
SlotNameListNumOfEntries(void)
{
  return slotNameList->entries;
}

CharPtr*
SlotNameListP(void)
{
  return slotNameList->listP;
}

Int
SlotNameListNumToSlot(Int listNum)
{
  // ErrFatalDisplayIf((listNum < 0) || (listNum >= slotNameList->entries), "SlotNameListNumToSlot");
  return slotNameList->listToSlot[listNum];
}

Int
SlotNameListSlotToListNum(Int slot)
{
  // ErrFatalDisplayIf((slot < 0) || (slot >= MaxSlot()), "SlotNameListSlotToListNum");
  return slotNameList->slotToList[slot];
}
