/*
 * timeForm: dialog to modify a projects time
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: timeForm.c,v 1.3 1999/06/03 12:04:50 ga58 Exp $
 *
 * $Log: timeForm.c,v $
 * Revision 1.3  1999/06/03 12:04:50  ga58
 *  - Version 3.32
 *
 * Revision 1.2  1999/02/28 16:36:52  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.4  1999/02/04 16:16:15  ga58
 *  - Version 3.26, new acceleration algorithm
 *
 * Revision 1.3  1999/01/31 19:05:20  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 */

#include "titraxAll.h"
#include "titraxID.h"
#include "callback.h"
#include "titraxPref.h"
#include "drawDigits.h"
#include "project.h"
#include "mainForm.h"
#include "accelerate.h"
#include "timeForm.h"

static Long minVal;
static Long maxVal;
static Long oldElapsed;
static Long newElapsed;

static FormPtr frm;

static void
TimeFormCheckScroll(void)
{
  FrmUpdateScrollers(frm,
							FrmGetObjectIndex(frm, timeFormUp),
							FrmGetObjectIndex(frm, timeFormDown),
							newElapsed < maxVal, newElapsed > minVal);
}

static void
TimeFormInit(void)
{
  Long minAmount, maxAmount;

  frm = FrmGetActiveForm();

  FrmDrawForm(frm);

  oldElapsed = TotalElapsed(Prefs()->editedSlot);
  newElapsed = oldElapsed;
  
  DrawDigits(newElapsed, timeFormTime, 45, stdFont);
  
  CalculateMinMaxChangeAmount(Prefs()->editedSlot, &minAmount, &maxAmount);
  
  minVal = oldElapsed + minAmount;
  maxVal = oldElapsed + maxAmount;
  
  TimeFormCheckScroll();

  Accel_Init(60);
}

Boolean
TimeFormHandleEvent(EventPtr event)
{
  Boolean handled;

  CALLBACK_PROLOGUE

  handled = false;

  Accel_Handle(event->eType == ctlRepeatEvent);

  switch (event->eType) {
  case ctlSelectEvent:
	 if (event->data.ctlEnter.controlID == timeFormCommit) {
		ChangeTime(Prefs()->editedSlot, newElapsed - oldElapsed);

		FrmGotoForm(mainForm);
		handled = true;
	 }

	 if (event->data.ctlEnter.controlID == timeFormCancel) {
		FrmGotoForm(mainForm);
		handled = true;
	 }
	 break;

  case ctlRepeatEvent: {
	 Int incr = Accel_Incr();

	 if (event->data.ctlRepeat.controlID == timeFormUp)
		newElapsed += incr;
	 if (event->data.ctlRepeat.controlID == timeFormDown)
		newElapsed -= incr;
	 
	 if (newElapsed < minVal)
		newElapsed = minVal;

	 if (newElapsed > maxVal)
		newElapsed = maxVal;

	 DrawDigits(newElapsed, timeFormTime, 45, stdFont);

	 TimeFormCheckScroll();
	 handled = false;
	 break;
  }

  case frmOpenEvent: {
	 TimeFormInit();
	 Prefs()->actualForm = timeForm;
	 handled = true;
	 break;
  }

  default:
	 handled = false;
  }

  CALLBACK_EPILOGUE

  return handled;
}
