/*
 * titraxCommands: common subroutines for the titrax main form
 *
 * Author: Bernd Esser <Bernd.Esser@testnetz.detecon.net>
 *
 * $Id: titraxCommands.c,v 1.2 1999/06/20 13:49:12 ga58 Exp $
 *
 * $Log: titraxCommands.c,v $
 * Revision 1.2  1999/06/20 13:49:12  ga58
 * Version 3.33
 *
 * Revision 1.1  1999/06/03 12:04:56  ga58
 *  - Version 3.32
 *
 */

#include "titraxAll.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "titraxPref.h"
#include "slotData.h"
#include "slotClear.h"
#include "slotNameList.h"
#include "project.h"
#include "titraxUtil.h"
#include "titraxCommands.h"

#define projectListButtonTextLen 40
static Char projectListButtonText[projectListButtonTextLen + 1];

static VoidPtr
GOP(Int ObjectID)
{
  FormPtr frm = FrmGetActiveForm();

  return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ObjectID));
}

void
UpdateProjectListButton(void)
{
  Int i;
  Int width = 0;
  FontID oldFont = FntSetFont(stdFont);
  Int num = SlotNameListNumOfEntries();
  CharPtr* list = SlotNameListP();
  ListPtr listP = GOP(mainFormProjectList);

  LstSetListChoices(listP, list, num);
  LstSetHeight(listP, num);

  if ((num == 0) || (ActiveSlot() == noSlotActive)) {
	 CtlSetLabel(GOP(mainFormProjectPopup), "None");
	 LstSetSelection(listP, noListSelection);
  }
  else {
	 StrNCopy(projectListButtonText, list[SlotNameListSlotToListNum(ActiveSlot())], 
				 projectListButtonTextLen);
	 TrimToLength(projectListButtonText, stdFont, 80);

	 CtlSetLabel(GOP(mainFormProjectPopup), projectListButtonText);
	 LstSetSelection(listP, SlotNameListSlotToListNum(ActiveSlot()));
  }

  for(i = 0; i < num; i++)
	 width = max(FntCharsWidth(list[i], StrLen(list[i])), width);

  listP->bounds.extent.x = width + 4;
  listP->bounds.topLeft.x = 160 - (width + 4) - 2;

  FntSetFont(oldFont);
}

void
UpdateSumTimeButton(void)
{
  const Int strLen = 4 * 8;
  FontID oldFont;
  Char time[strLen + 1];
  ULong sum = SumElapsed();

  oldFont = FntSetFont(boldFont);

  StrNCopy(time, TimeToString(sum, time, strLen), strLen);

  // 42 is the width of the sumtime button in titrax.rcp
  ExpandLeftToLength(time, strLen, 42);

  CtlSetLabel(GOP(mainFormTimeSum), time);
  
  FntSetFont(oldFont);
}

void
UpdateSlot(Int slot)
{
  if (FrmGetActiveFormID() != mainForm)
	 return;

  if (slot == noSlotActive)
	 return;

  if (slot < Prefs()->firstVisibleSlot)
	 return;

  if (slot >= Prefs()->firstVisibleSlot + VISSLOT)
	 return;

  TblMarkRowInvalid(GOP(mainFormTable), slot - Prefs()->firstVisibleSlot);
  TblRedrawTable(GOP(mainFormTable));
}

void
UpdateActiveSlot(void)
{
  UpdateSlot(ActiveSlot());
}

void 
UpdateAllSlots(void)
{
  TablePtr tableP = GOP(mainFormTable);

  if (FrmGetActiveFormID() != mainForm)
	 return;

  TblMarkTableInvalid(tableP);
  TblRedrawTable(tableP);
}

void
StopAndUpdate(Int slot)
{
  if (slot == noSlotActive)
	 return;

  Stop(slot);

  UpdateProjectListButton();
  UpdateSumTimeButton();
  UpdateSlot(slot);
}

void
StartAndUpdate(Int slot)
{
  Int oldActiveSlot = ActiveSlot();

  Start(slot);

  UpdateProjectListButton();
  UpdateSumTimeButton();
  UpdateSlot(oldActiveSlot);
  UpdateSlot(slot);
}

void
DoCommand(enum commands commandCode, Int slot)
{
  switch (commandCode) {
  case saveAll:
	 Prefs()->editedSlot = noSlotActive;
	 FrmGotoForm(saveForm);
	 break;
  case clearAll:
	 Prefs()->editedSlot = noSlotActive;
	 FrmPopupForm(clearForm);
	 break;
  case removeAll:
	 if (FrmAlert(titraxRemoveAllAlert) == 0) {
		Stop(ActiveSlot());
		SlotRemoveAll();
		SlotNameListUpdate();
		UpdateProjectListButton();
		UpdateSumTimeButton();
		UpdateAllSlots();
	 }
	 break;
  case changeTime:
  case changeTime2:
	 if ((SlotData(slot)->logging != dontLog) && Prefs()->shiftWarnings)
		(void)FrmAlert(titraxMainFormDateBookAlert);
	 Prefs()->editedSlot = slot;
	 FrmPopupForm(timeForm);
	 break;
  case shiftTime:
  case shiftTime2:
	 if ((SlotData(slot)->logging != dontLog) && Prefs()->shiftWarnings)
		(void)FrmAlert(titraxMainFormDateBookAlert);
	 Prefs()->editedSlot = slot;
	 FrmPopupForm(shiftForm);
	 break;
  case clear:
  case clear2:
	 Prefs()->editedSlot = slot;
	 FrmPopupForm(clearForm);
	 break;
  case remove:
  case remove2:
	 if (FrmAlert(titraxRemoveAlert) == 0) {
		Stop(slot);
		SlotRemove(slot);
		SlotNameListUpdate();
		UpdateProjectListButton();
		UpdateSumTimeButton();
		UpdateSlot(slot);
	 }
	 break;
  case changeName:
  case changeName2:
	 Prefs()->editedSlot = slot;
	 FrmGotoForm(editForm);
	 break;
  case editNote:
	 Prefs()->editedSlot = slot;
	 FrmGotoForm(noteForm);
	 break;
  case startStop:
	 if (SlotIsEmpty(slot))
		DoCommand(changeName, slot);
	 else {
		if (slot != ActiveSlot()) 
		  StartAndUpdate(slot);
		else
		  StopAndUpdate(slot);
	 }
	 break;
  default:
  }
}
		
