/*
 * titraxHist.c
 *
 * Titrax log-database subroutines
 *
 * $Id: titraxHist.c,v 1.1 1999/02/21 13:41:57 ga58 Exp $
 *
 * $Log: titraxHist.c,v $
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.2  1999/01/24 20:23:11  ga58
 *  - Beta version 3.23
 *
 * Revision 1.1  1999/01/23 16:02:12  ga58
 *  - checkin before removing all row-constructs
 *
 */

#pragma pack(2)

#include "titraxAll.h"
#include "titraxConst.h"
#include "callback.h"
#include "titraxDB.h"
#include "titraxNote.h"

static DmOpenRef TitraxDB;

#define titraxDBName "TitraxHistDB"
#define titraxDBType 'Hist'

typedef struct {
  char name[MAXTEXT+1];
  ULong start;
  ULong stop;
  Char note[1];
} histType;

typedef histType* histTypePtr;

/*
 * Database operations
 * 
 * These routines do NOT perform any error-checking
 */
void
CreateHistDB(void)
{
  Err err;
  LocalID dbID;

  /* check for a existing database */
  dbID = DmFindDatabase(0, titraxDBName);
  if (dbID != 0)
	 DmDeleteDatabase(0, dbID);

  err = DmCreateDatabase(0, titraxDBName, titraxAppType, titraxDBType, false);
  ErrFatalDisplayIf(err, "Could not create database.");

  TitraxDB = DmOpenDatabaseByTypeCreator(titraxDBType, titraxAppType, dmModeReadWrite);
  ErrFatalDisplayIf(TitraxDB == 0, "Could not open database.");
}

void
OpenHistDB(void)
{
  TitraxDB = DmOpenDatabaseByTypeCreator(titraxDBType, titraxAppType, dmModeReadWrite);
  if (TitraxDB == 0)
	 CreateHistDB();
}

void
CloseHistDB(void)
{
  DmCloseDatabase(TitraxDB);
}
Int
HistCompareRecords(histTypePtr r1, histTypePtr r2, Int extra,
						 SortRecordInfoPtr info1, SortRecordInfoPtr info2,
						 VoidHand appInfoH)
{
  Int result;

  CALLBACK_PROLOGUE

  if (r1->start < r2->start)
	 result = -1;
  else if (r1->start > r2->start)
	 result = 1;
  else {
	 if (r1->stop < r2->stop)
		result = -1;
	 else if (r1->stop > r2->stop)
		result = 1;
	 else
		result = 0;
  }

  CALLBACK_EPILOGUE

  return result;
}

void
AddLogEntry(Int slot, ULong start, ULong stop)
{
  UInt idx;
  ULong size;
  Char name[MAXTEXT+1];
  VoidHand recH;
  histTypePtr recP;

  OpenHistDB();

  size = sizeof(histType) + NoteLen(slot);

  recH = DmNewHandle(TitraxDB, size);
  if (recH == NULL)
	 ErrDisplay("HistCreateRecord: could not create handle.");

  recP = (histTypePtr)MemHandleLock(recH);
  DmSet(recP, 0, size, 0);

  Name(slot, name, MAXTEXT);

  DmStrCopy(recP, 0, name);
  DmWrite(recP, (void*)&(recP->start) - (void*)recP, &start, sizeof(start));
  DmWrite(recP, (void*)&(recP->stop) - (void*)recP, &stop, sizeof(stop));
  NoteDmStrCopy(slot, (CharPtr)recP, (void*)&(recP->note[0]) - (void*)recP);
  
  idx = DmFindSortPosition(TitraxDB, recP, NULL, (DmComparF*)HistCompareRecords, 0);

  MemPtrUnlock(recP);

  if (DmAttachRecord(TitraxDB, &idx, (Handle)recH, 0)) {
	 MemHandleFree(recH);
	 ErrDisplay("AddLogEntry: could not attach record.");
  }

  CloseHistDB();
}

