/*
 * Titrax preferences subroutines
 *
 * $Id: titraxPref.c,v 1.4 1999/06/03 12:04:57 ga58 Exp $
 *
 * $Log: titraxPref.c,v $
 * Revision 1.4  1999/06/03 12:04:57  ga58
 *  - Version 3.32
 *
 * Revision 1.3  1999/02/28 16:36:58  ga58
 *  - Version 3.31
 *
 * Revision 1.2  1999/02/27 17:39:10  ga58
 *  - Version 3.30
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.5  1999/01/31 19:05:24  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.4  1999/01/30 19:08:32  ga58
 *  - Version 3.24. Dynamically resizable number of project slots.
 *
 * Revision 1.3  1999/01/24 20:23:15  ga58
 *  - Beta version 3.23
 *
 * Revision 1.2  1999/01/17 20:02:31  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 * Revision 1.1  1998/12/30 10:27:57  ga58
 * Last changes for the new layout
 *
 * Revision 1.2  1998/12/30 09:10:41  ga58
 * I have changed the startup code, so that upgrades without data losses
 * should be possible.
 *
 * Revision 1.1  1998/12/29 10:10:26  ga58
 * Titrax v3.17 in the new directory
 *
 * Revision 3.9  1998/12/27 18:15:07  ga58
 * I finally managed to create a new version. Mostly user interface changes.
 *
 * Revision 3.8  1998/10/17 14:06:03  ga58
 *  - changed the drag-n-drop mechanism to use a minimum drag length
 *
 * Revision 3.7  1998/04/27 06:36:47  ga58
 *  - the width of the command popup window and the project list window are now
 *    calculated dynamically.
 * CVS:
 * CVS:
 *
 * Revision 3.6  1998/04/26 09:03:29  ga58
 *  - finished the conversion to a shared library version
 * CVS:
 * CVS:
 *
 * Revision 3.5  1998/04/21 05:20:01  ga58
 *  - new functions SumElapsed. This is a preliminary step for implementing
 *    project flags. The first flag will be a ignored-flag. Projects with
 *    this flag set, will be ignored in the sum and in the sorting process.
 * CVS:
 * CVS:
 *
 * Revision 3.4  1998/04/19 15:44:30  ga58
 * This is the first branch of the 3.x series of titrax. Most of the new
 * features and code changes are described in the NEWS file.
 * CVS:
 * CVS:
 *
 * Revision 2.8  1998/04/13 09:22:33  ga58
 *  - added a new option: Clear note in datebook mode
 * CVS:
 * CVS:
 *
 * Revision 2.7  1998/04/12 09:31:09  ga58
 *  - a lot of minor changes in the UI
 *  - a lot of minor changes in the code
 *  - added new options
 * CVS:
 * CVS:
 *
 * Revision 2.6  1998/04/10 16:54:10  ga58
 * code cleanups in some modules
 * added the list-code to titraxDB
 * modularized mainForm
 * removed Elapsed and StartedAt from titraxDB and integrated them into
 * titraxPref
 * reworked TextIsEmpty and NoteIsEmpty in titraxDB
 * tried to make the code faster
 * CVS:
 * CVS:
 *
 * Revision 2.5  1998/04/05 16:23:55  ga58
 * removed LastCall and SetLastCall
 * CVS:
 * CVS:
 *
 * Revision 2.4  1998/04/04 15:26:35  ga58
 * version 2.4
 * CVS:
 * CVS:
 *
 * Revision 1.1  1998/04/04 15:24:33  ga58
 * first version in CVS
 * CVS:
 *
 * Revision 1.1.1.1  1998/04/04 15:04:07  ga58
 * first version
 *
 * Revision 1.9  1998/04/04 14:41:06  ga58
 * *** empty log message ***
 *
 * Revision 1.8  1998/03/24 17:23:32  ga58
 * *** empty log message ***
 *
 * Revision 1.7  1998/03/24 07:50:50  ga58
 * *** empty log message ***
 *
 * Revision 1.6  1998/03/21 16:06:31  ga58
 * *** empty log message ***
 *
 * Revision 1.5  1998/02/13 15:26:55  ga58
 * Increased the number of available project-slots from 11 to 30
 *
 * Revision 1.4  1998/01/31 15:48:48  ga58
 * Added the option to toggle the display format for times.
 *
 * Revision 1.3  1997/11/23 20:08:59  ga58
 * *** empty log message ***
 *
 * Revision 1.2  1997/11/15 17:59:29  ga58
 * *** empty log message ***
 *
 * Revision 1.1  1997/11/14 17:51:04  ga58
 * Initial revision
 *
 */

#pragma pack(2)

#include "titraxAll.h"
#include "titraxID.h"
#include "titraxConst.h"
#include "titraxPref.h"
  
static VoidHand prefHand;
static TitraxPreferenceType* prefP;

#define titraxVersionNum 57

void 
CreatePrefs(void)
{
  prefHand = MemHandleNew(sizeof(TitraxPreferenceType));
  ErrFatalDisplayIf(prefHand == 0, "Out of memory for preferences.");

  prefP = (TitraxPreferenceType*)MemHandleLock(prefHand);

  MemSet(prefP, sizeof(TitraxPreferenceType), 0);
}

void
ClearPrefs(void)
{
  prefP->editedSlot = noSlotActive;
  prefP->actualForm = mainForm;
  prefP->maxSlot = DEFSLOT;
  prefP->timeFormat = noDays;
  prefP->firstVisibleSlot = 0;
  prefP->saveSummaryTo = datebook;
  prefP->shiftWarnings = true;
  prefP->menuHack = true;
  prefP->newUI = false;
  prefP->clearTime = true;
  prefP->clearNote = true;
  prefP->summary.headerLine = true;
  prefP->summary.fieldDescr = true;
  prefP->summary.inclName = true;
  prefP->summary.inclTime = true;
  prefP->summary.inclNote = true;
  prefP->summary.quoteFields = true;
  prefP->summary.fieldDelimiter = Comma;
}

void
ReadPrefs(void)
{
  Word prefSize = sizeof(TitraxPreferenceType);

  CreatePrefs();
  ClearPrefs();

  (void)PrefGetAppPreferences((ULong)titraxAppType, 0, prefP, &prefSize, true);
}

void
DeletePrefs()
{
  if (MemHandleUnlock(prefHand) != 0)
	 return;
  if (MemHandleFree(prefHand) != 0)
	 return;
}

void
WritePrefs(void)
{
  PrefSetAppPreferences((ULong)titraxAppType, 0, titraxVersionNum, prefP, sizeof(TitraxPreferenceType), true);

  DeletePrefs();
}

TitraxPreferenceType*
Prefs(void)
{
  return prefP;
}
