/*
 * Titrax utility subroutines
 *
 * $Id: titraxUtil.c,v 1.2 1999/02/28 16:37:00 ga58 Exp $
 *
 * $Log: titraxUtil.c,v $
 * Revision 1.2  1999/02/28 16:37:00  ga58
 *  - Version 3.31
 *
 * Revision 1.1  1999/02/21 13:41:57  ga58
 *  - Version 3.29
 *
 * Revision 1.4  1999/01/31 19:05:26  ga58
 *  - Version 3.25. New layout for the time boxes and version info for the
 *    databases.
 *
 * Revision 1.3  1999/01/24 20:23:17  ga58
 *  - Beta version 3.23
 *
 * Revision 1.2  1999/01/17 20:02:33  ga58
 *  - nearly complete rewrite of titrax and all its modules
 *
 * Revision 1.1  1998/12/30 10:27:57  ga58
 * Last changes for the new layout
 *
 * Revision 1.1  1998/12/29 10:10:26  ga58
 * Titrax v3.17 in the new directory
 *
 * Revision 3.8  1998/05/15 16:20:22  ga58
 *  - no visible changes
 *
 * Revision 3.7  1998/04/27 06:36:47  ga58
 *  - the width of the command popup window and the project list window are now
 *    calculated dynamically.
 * CVS:
 * CVS:
 *
 * Revision 3.6  1998/04/26 09:03:29  ga58
 *  - finished the conversion to a shared library version
 * CVS:
 * CVS:
 *
 * Revision 3.5  1998/04/21 05:20:01  ga58
 *  - new functions SumElapsed. This is a preliminary step for implementing
 *    project flags. The first flag will be a ignored-flag. Projects with
 *    this flag set, will be ignored in the sum and in the sorting process.
 * CVS:
 * CVS:
 *
 * Revision 3.4  1998/04/19 15:44:30  ga58
 * This is the first branch of the 3.x series of titrax. Most of the new
 * features and code changes are described in the NEWS file.
 * CVS:
 * CVS:
 *
 * Revision 2.6  1998/04/12 09:31:10  ga58
 *  - a lot of minor changes in the UI
 *  - a lot of minor changes in the code
 *  - added new options
 * CVS:
 * CVS:
 *
 * Revision 2.5  1998/04/10 16:54:11  ga58
 * code cleanups in some modules
 * added the list-code to titraxDB
 * modularized mainForm
 * removed Elapsed and StartedAt from titraxDB and integrated them into
 * titraxPref
 * reworked TextIsEmpty and NoteIsEmpty in titraxDB
 * tried to make the code faster
 * CVS:
 * CVS:
 *
 * Revision 2.4  1998/04/04 15:26:36  ga58
 * version 2.4
 * CVS:
 * CVS:
 *
 * Revision 1.1  1998/04/04 15:24:33  ga58
 * first version in CVS
 * CVS:
 *
 * Revision 1.1.1.1  1998/04/04 15:04:07  ga58
 * first version
 *
 * Revision 2.1  1998/03/24 17:23:32  ga58
 * *** empty log message ***
 *
 * Revision 2.0  1998/03/24 08:55:56  ga58
 * *** empty log message ***
 *
 * Revision 1.7  1998/02/14 12:16:50  ga58
 * fixed a bug in SaveData. Now uses StrNCat, so titrax won't run on
 * PalmOS 1.x devices
 *
 * Revision 1.6  1998/01/31 15:48:48  ga58
 * Added the option to toggle the display format for times.
 *
 * Revision 1.5  1997/12/06 13:42:04  ga58
 * - total time is no saved into the memopad note
 *
 * Revision 1.4  1997/11/23 20:08:59  ga58
 * *** empty log message ***
 *
 * Revision 1.3  1997/11/23 15:30:38  ga58
 * - fixed a off-by-one bug in ReadNote
 *
 * Revision 1.2  1997/11/15 17:59:29  ga58
 * *** empty log message ***
 *
 * Revision 1.1  1997/11/14 17:51:04  ga58
 * Initial revision
 *
 */

#include "titraxAll.h"
#include "titraxConst.h"
#include "titraxPref.h"

void
TrimToLength(CharPtr text, FontID font, Int len)
{
  FontID oldFont = FntSetFont(font);

  if (FntCharsWidth(text, StrLen(text)) > len) {
	 while (FntCharsWidth(text, StrLen(text)) > (len - 6))
		text[StrLen(text)-1] = 0;
	 StrCat(text, "\205");
  }

  FntSetFont(oldFont);
}

void 
ExpandLeftToLength(CharPtr text, Int strLen, Int len)
{
  Int i;
  CharPtr oldP;
  VoidHand oldH;

  oldH = MemHandleNew(strLen);
  oldP = MemHandleLock(oldH);

  StrCopy(oldP, text);

  StrCopy(text, "");
  
  for(i = FntCharsWidth(oldP, StrLen(oldP)); 
		i <= len - FntCharWidth(' '); 
		i+=FntCharWidth(' '))
	 StrNCat(text, " ", strLen);

  StrNCat(text, oldP, strLen);

  MemHandleUnlock(oldH);
  MemHandleFree(oldH);
}

/*
 * Time to string conversion
 */
CharPtr
TimeToString(ULong t, CharPtr str, Int strLen)
{
  Long min;
  Long hour;
  Long day;

  Char tmpStr[20];

  if (Prefs()->timeFormat == days) {
	 day = t / 86400;
	 t %= 86400;
  }
  else
	 day = 0;

  hour = t / 3600;
  t %= 3600;
  min = t / 60;

  StrNCopy(str, "", strLen);

  if (Prefs()->timeFormat == days) {
	 if (day > 0) {
		StrIToA(tmpStr, day);
		StrNCat(str, tmpStr, strLen);
		StrNCat(str, ":", strLen);
	 }
	 
	 if ((day > 0) && (hour < 10))
		StrNCat(str, "0", strLen);
  }

  StrIToA(tmpStr, hour);
  StrNCat(str, tmpStr, strLen);
  StrNCat(str, ":", strLen);

  if (min < 10)
	 StrNCat(str, "0", strLen);
  StrIToA(tmpStr, min);
  StrNCat(str, tmpStr, strLen);

  return str;
}

void 
TimeString(CharPtr str, ULong time)
{
  DateTimeType dateTime;

  TimSecondsToDateTime(time, &dateTime);
  TimeToAscii(dateTime.hour, dateTime.minute, PrefGetPreference(prefTimeFormat), str);
}

void
DateString(CharPtr str, ULong time)
{
  DateTimeType dateTime;

  TimSecondsToDateTime(time, &dateTime);
  DateToAscii(dateTime.month, dateTime.day, dateTime.year, PrefGetPreference(prefDateFormat), str);
}

  
